@@ -32,6 +32,7 @@
 
 #include "ctkconfig.h"
 #include "ctkhelp.h"
+#include "ctkutils.h"
 
 #include <string.h>
 #include <stdlib.h>
@@ -57,7 +58,7 @@ static const char *__color_curve_help = "The color curve graph changes to "
 "sliders.";
 
 static void
-option_menu_changed         (GtkOptionMenu *, gpointer);
+color_channel_changed         (GtkComboBox *, gpointer);
 
 static void
 set_button_sensitive        (GtkButton *);
@@ -83,12 +84,17 @@ flush_attribute_channel_values (CtkColorCorrection *, gint, gint);
 static void
 ctk_color_correction_class_init(CtkColorCorrectionClass *);
 
+static void ctk_color_correction_finalize(GObject *);
+
 static void
 apply_parsed_attribute_list(CtkColorCorrection *, ParsedAttribute *);
 
 static gboolean
 do_confirm_countdown (gpointer);
 
+static void callback_palette_update(GObject *object, CtrlEvent *event,
+                                    gpointer user_data);
+
 static void
 update_confirm_text  (CtkColorCorrection *);
 
@@ -111,18 +117,18 @@ static guint signals[LAST_SIGNAL] = { 0 };
 
 #define DEFAULT_CONFIRM_COLORCORRECTION_TIMEOUT 10
 
-#define CREATE_COLOR_ADJUSTMENT(adj, attr, min, max)          \
-{                                                             \
-    gdouble _step_incr, _page_incr, _def;                     \
-                                                              \
-    _step_incr = ((gdouble)((max) - (min)))/250.0;            \
-    _page_incr = ((gdouble)((max) - (min)))/25.0;             \
-                                                              \
-    _def = get_attribute_channel_value(ctk_color_correction,  \
-                                       (attr), ALL_CHANNELS); \
-                                                              \
-    (adj) = gtk_adjustment_new(_def, (min), (max),            \
-                               _step_incr, _page_incr, 0.0);  \
+#define CREATE_COLOR_ADJUSTMENT(adj, attr, min, max)                         \
+{                                                                            \
+    gdouble _step_incr, _page_incr, _def;                                    \
+                                                                             \
+    _step_incr = ((gdouble)((max) - (min)))/250.0;                           \
+    _page_incr = ((gdouble)((max) - (min)))/25.0;                            \
+                                                                             \
+    _def = get_attribute_channel_value(ctk_color_correction,                 \
+                                       (attr), ALL_CHANNELS);                \
+                                                                             \
+    (adj) = GTK_ADJUSTMENT(gtk_adjustment_new(_def, (min), (max),            \
+                                              _step_incr, _page_incr, 0.0)); \
 }
 
 GType ctk_color_correction_get_type(
@@ -154,10 +160,13 @@ GType ctk_color_correction_get_type(
 }
 
 
+
 static void
 ctk_color_correction_class_init(CtkColorCorrectionClass
                                 *ctk_color_correction_class)
 {
+    GObjectClass *gobject_class = (GObjectClass *)ctk_color_correction_class;
+
     signals[CHANGED] =
         g_signal_new("changed",
                      G_OBJECT_CLASS_TYPE(ctk_color_correction_class),
@@ -166,23 +175,79 @@ ctk_color_correction_class_init(CtkColorCorrectionClass
                      NULL, NULL,
                      g_cclosure_marshal_VOID__VOID,
                      G_TYPE_NONE, 0);
+    gobject_class->finalize = ctk_color_correction_finalize;
 }
 
 
-GtkWidget* ctk_color_correction_new(NvCtrlAttributeHandle *handle,
-                                    CtkConfig *ctk_config, ParsedAttribute *p,
+
+static void ctk_color_correction_finalize(GObject *object)
+{
+    CtkColorCorrection *ctk_color_correction = CTK_COLOR_CORRECTION(object);
+    CtrlTarget *ctrl_target = ctk_color_correction->ctrl_target;
+
+    if (ctk_color_correction->confirm_timer) {
+        /*
+         * This situation comes, if user perform VT-switching
+         * without confirmation of color-correction settings.
+         */
+        gint attr, ch;
+        unsigned int channels = 0;
+        unsigned int attributes = 0;
+
+        /* kill the timer */
+        g_source_remove(ctk_color_correction->confirm_timer);
+        ctk_color_correction->confirm_timer = 0;
+
+        /*
+         * Reset color settings to previous state,
+         * since user did not confirm settings yet.
+         */
+        for (attr = CONTRAST_INDEX; attr <= GAMMA_INDEX; attr++) {
+            for (ch = RED; ch <= ALL_CHANNELS_INDEX; ch++) {
+                    /* Check for attribute channel value change. */
+                    int index = attr - CONTRAST_INDEX;
+
+                    ctk_color_correction->cur_slider_val[index][ch] =
+                        ctk_color_correction->prev_slider_val[index][ch];
+
+                    attributes |= (1 << attr);
+                    channels |= (1 << ch);
+            }
+        }
+
+        NvCtrlSetColorAttributes(ctrl_target,
+                                 ctk_color_correction->cur_slider_val[CONTRAST],
+                                 ctk_color_correction->cur_slider_val[BRIGHTNESS],
+                                 ctk_color_correction->cur_slider_val[GAMMA],
+                                 attributes | channels);
+    }
+
+    g_signal_handlers_disconnect_matched(G_OBJECT(ctk_color_correction->ctk_event),
+                                         G_SIGNAL_MATCH_DATA,
+                                         0,
+                                         0,
+                                         NULL,
+                                         NULL,
+                                         (gpointer) ctk_color_correction);
+
+
+}
+
+
+
+GtkWidget* ctk_color_correction_new(CtrlTarget *ctrl_target,
+                                    CtkConfig *ctk_config,
+                                    ParsedAttribute *p,
                                     CtkEvent *ctk_event)
 {
     GObject *object;
     CtkColorCorrection *ctk_color_correction;
     GtkRequisition requisition;
 
-    GtkWidget *menu;
     GtkWidget *image;
     GtkWidget *label;
     GtkWidget *scale;
     GtkWidget *curve;
-    GtkWidget *menu_item;
     GtkWidget *alignment;
     GtkWidget *mainhbox;
     GtkWidget *leftvbox;
@@ -192,13 +257,19 @@ GtkWidget* ctk_color_correction_new(NvCtrlAttributeHandle *handle,
     GtkWidget *button, *confirm_button, *confirm_label;
     GtkWidget *widget;
     GtkWidget *hsep;
+    GtkWidget *center_alignment;
     GtkWidget *eventbox;
+    GtkWidget *combo_box;
+    GtkListStore *store;
+    GtkTreeIter iter;
+    GtkCellRenderer *renderer;
 
     object = g_object_new(CTK_TYPE_COLOR_CORRECTION, NULL);
 
     ctk_color_correction = CTK_COLOR_CORRECTION(object);
-    ctk_color_correction->handle = handle;
+    ctk_color_correction->ctrl_target = ctrl_target;
     ctk_color_correction->ctk_config = ctk_config;
+    ctk_color_correction->ctk_event = ctk_event;
     ctk_color_correction->confirm_timer = 0;
     ctk_color_correction->confirm_countdown =
         DEFAULT_CONFIRM_COLORCORRECTION_TIMEOUT;
@@ -235,76 +306,53 @@ GtkWidget* ctk_color_correction_new(NvCtrlAttributeHandle *handle,
     gtk_box_pack_start(GTK_BOX(vbox), label, FALSE, FALSE, 0);
     gtk_container_add(GTK_CONTAINER(alignment), vbox);
 
-    menu = gtk_menu_new();
-    
-    menu_item = gtk_menu_item_new();
-    gtk_menu_shell_append(GTK_MENU_SHELL(menu), menu_item);
+    store = gtk_list_store_new(2, GDK_TYPE_PIXBUF, G_TYPE_STRING);
 
-    hbox = gtk_hbox_new(FALSE, 0);
-    gtk_container_add(GTK_CONTAINER(menu_item), hbox);
+    gtk_list_store_append(store, &iter);
+    gtk_list_store_set(store, &iter,
+                       0, gdk_pixbuf_new_from_xpm_data(rgb_xpm),
+                       1, "All Channels", -1);
+    gtk_list_store_append(store, &iter);
+    gtk_list_store_set(store, &iter,
+                       0, gdk_pixbuf_new_from_xpm_data(red_xpm),
+                       1, "Red", -1);
+    gtk_list_store_append(store, &iter);
+    gtk_list_store_set(store, &iter,
+                       0, gdk_pixbuf_new_from_xpm_data(green_xpm),
+                       1, "Green", -1);
+    gtk_list_store_append(store, &iter);
+    gtk_list_store_set(store, &iter,
+                       0, gdk_pixbuf_new_from_xpm_data(blue_xpm),
+                       1, "Blue", -1);
 
-    label = gtk_label_new("All Channels");
-    image = gtk_image_new_from_pixbuf(gdk_pixbuf_new_from_xpm_data(rgb_xpm));
+    combo_box = gtk_combo_box_new_with_model(GTK_TREE_MODEL(store));
 
-    gtk_box_pack_start(GTK_BOX(hbox), image, FALSE, FALSE, 0);
-    gtk_box_pack_start(GTK_BOX(hbox), label, FALSE, FALSE, 10);
+    renderer = gtk_cell_renderer_pixbuf_new();
+    gtk_cell_layout_pack_start(GTK_CELL_LAYOUT(combo_box), renderer, FALSE);
+    gtk_cell_layout_set_attributes(GTK_CELL_LAYOUT(combo_box), renderer,
+                                   "pixbuf", 0, NULL);
 
+    renderer = gtk_cell_renderer_text_new();
+    gtk_cell_layout_pack_start(GTK_CELL_LAYOUT(combo_box), renderer, FALSE);
+    gtk_cell_layout_set_attributes(GTK_CELL_LAYOUT(combo_box), renderer,
+                                   "text", 1, NULL);
 
-    menu_item = gtk_menu_item_new();
-    gtk_menu_shell_append(GTK_MENU_SHELL(menu), menu_item);
+    gtk_combo_box_set_active(GTK_COMBO_BOX(combo_box), 0);
 
-    hbox = gtk_hbox_new(FALSE, 0);
-    gtk_container_add(GTK_CONTAINER(menu_item), hbox);
-
-    label = gtk_label_new ("Red");
-    image = gtk_image_new_from_pixbuf(gdk_pixbuf_new_from_xpm_data(red_xpm));
-
-    gtk_box_pack_start(GTK_BOX(hbox), image, FALSE, FALSE, 0);
-    gtk_box_pack_start(GTK_BOX(hbox), label, FALSE, FALSE, 10);
-
-
-    menu_item = gtk_menu_item_new();
-    gtk_menu_shell_append(GTK_MENU_SHELL(menu), menu_item);
-
-    hbox = gtk_hbox_new(FALSE, 0);
-    gtk_container_add(GTK_CONTAINER(menu_item), hbox);
-
-    label = gtk_label_new("Green");
-    image = gtk_image_new_from_pixbuf(gdk_pixbuf_new_from_xpm_data(green_xpm));
-
-    gtk_box_pack_start(GTK_BOX(hbox), image, FALSE, FALSE, 0);
-    gtk_box_pack_start(GTK_BOX(hbox), label, FALSE, FALSE, 10);
-
-
-    menu_item = gtk_menu_item_new();
-    gtk_menu_shell_append(GTK_MENU_SHELL(menu), menu_item);
-
-    hbox = gtk_hbox_new(FALSE, 0);
-    gtk_container_add(GTK_CONTAINER(menu_item), hbox);
-
-    label = gtk_label_new("Blue");
-    image = gtk_image_new_from_pixbuf(gdk_pixbuf_new_from_xpm_data(blue_xpm));
-
-    gtk_box_pack_start(GTK_BOX(hbox), image, FALSE, FALSE, 0);
-    gtk_box_pack_start(GTK_BOX(hbox), label, FALSE, FALSE, 10);
-
-
-    ctk_color_correction->option_menu = gtk_option_menu_new ();
-    gtk_option_menu_set_menu
-        (GTK_OPTION_MENU(ctk_color_correction->option_menu), menu);
+    ctk_color_correction->color_channel = combo_box;
 
     gtk_box_pack_start(GTK_BOX(vbox),
-                       ctk_color_correction->option_menu,
+                       ctk_color_correction->color_channel,
                        FALSE, FALSE, 0); 
 
-    g_object_set_data(G_OBJECT(ctk_color_correction->option_menu),
+    g_object_set_data(G_OBJECT(ctk_color_correction->color_channel),
                       "color_channel", GINT_TO_POINTER(ALL_CHANNELS));
-    
-    g_signal_connect(G_OBJECT(ctk_color_correction->option_menu), "changed",
-                     G_CALLBACK(option_menu_changed),
+
+    g_signal_connect(G_OBJECT(ctk_color_correction->color_channel), "changed",
+                     G_CALLBACK(color_channel_changed),
                      (gpointer) ctk_color_correction);
 
-    ctk_config_set_tooltip(ctk_config, ctk_color_correction->option_menu,
+    ctk_config_set_tooltip(ctk_config, ctk_color_correction->color_channel,
                            __active_color_help);
     /*
      * Gamma curve: BOTTOM - LEFT
@@ -316,12 +364,13 @@ GtkWidget* ctk_color_correction_new(NvCtrlAttributeHandle *handle,
     alignment = gtk_alignment_new(0, 0, 1.0, 1.0);
     gtk_box_pack_start(GTK_BOX(leftvbox), alignment, TRUE, TRUE, 0);
 
-    curve = ctk_curve_new(handle, GTK_WIDGET(ctk_color_correction));
+    curve = ctk_curve_new(ctrl_target, GTK_WIDGET(ctk_color_correction));
     eventbox = gtk_event_box_new();
     gtk_container_add(GTK_CONTAINER(eventbox), curve);
     gtk_container_add(GTK_CONTAINER(alignment), eventbox);
 
     ctk_config_set_tooltip(ctk_config, eventbox, __color_curve_help);
+    ctk_color_correction->curve = curve;
 
     /*
      * Reset button: BOTTOM - RIGHT (see below)
@@ -351,6 +400,7 @@ GtkWidget* ctk_color_correction_new(NvCtrlAttributeHandle *handle,
 
     ctk_color_correction->confirm_label = confirm_label;
     ctk_color_correction->confirm_button = confirm_button;
+    ctk_color_correction->reset_button = button;
     ctk_config_set_tooltip(ctk_config, eventbox, __confirm_button_help);
     ctk_config_set_tooltip(ctk_config, button, __resest_button_help);
 
@@ -468,10 +518,36 @@ GtkWidget* ctk_color_correction_new(NvCtrlAttributeHandle *handle,
     gtk_container_add(GTK_CONTAINER(alignment), hbox);
     gtk_box_pack_start(GTK_BOX(object), alignment, TRUE, TRUE, 0);
 
+    /* external update notification label */
+
+    center_alignment = gtk_alignment_new(0.5, 0.5, 0, 0);
+    gtk_box_pack_start(GTK_BOX(vbox), center_alignment, TRUE, TRUE, 0);
+
+    hbox = gtk_hbox_new(FALSE, 10);
+    gtk_container_add(GTK_CONTAINER(center_alignment), hbox);
+
+    label = gtk_label_new("Warning: The color settings have been changed "
+                          "outside of nvidia-settings so the current slider "
+                          "values may be incorrect.");
+    gtk_label_set_line_wrap(GTK_LABEL(label), TRUE);
+
+    image = gtk_image_new_from_stock(GTK_STOCK_DIALOG_WARNING,
+                                     GTK_ICON_SIZE_BUTTON);
+
+    gtk_box_pack_start(GTK_BOX(hbox), image, FALSE, FALSE, 0);
+    gtk_box_pack_start(GTK_BOX(hbox), label, FALSE, FALSE, 0);
+
+    ctk_color_correction->warning_container = hbox;
+
+
     /* finally, show the widget */
 
     gtk_widget_show_all(GTK_WIDGET(object));
 
+    /* except the external color change update warning */
+
+    gtk_widget_hide(ctk_color_correction->warning_container);
+
     /*
      * lock the size of the confirm button, so that it is not resized
      * when we change the button text later.
@@ -484,21 +560,26 @@ GtkWidget* ctk_color_correction_new(NvCtrlAttributeHandle *handle,
      *  "%d Seconds to Confirm"
      */
 
-    gtk_widget_size_request(ctk_color_correction->confirm_label,
-                            &requisition);
+    ctk_widget_get_preferred_size(ctk_color_correction->confirm_label,
+                                  &requisition);
     gtk_widget_set_size_request(ctk_color_correction->confirm_label,
                                 requisition.width, -1);
 
+    g_signal_connect(G_OBJECT(ctk_event),
+                     CTK_EVENT_NAME(NV_CTRL_PALETTE_UPDATE_EVENT),
+                     G_CALLBACK(callback_palette_update),
+                     (gpointer) ctk_color_correction);
+
     return GTK_WIDGET(object);
 }
 
 
-static void option_menu_changed(
-    GtkOptionMenu *option_menu,
+static void color_channel_changed(
+    GtkComboBox *combo_box,
     gpointer user_data
 )
 {
-    GtkObject *adjustment;
+    GtkAdjustment *adjustment;
     gint history, attribute, channel;
     CtkColorCorrection *ctk_color_correction;
     gfloat value;
@@ -506,7 +587,7 @@ static void option_menu_changed(
 
     ctk_color_correction = CTK_COLOR_CORRECTION(user_data);
 
-    history = gtk_option_menu_get_history(option_menu);
+    history = gtk_combo_box_get_active(combo_box);
 
     switch (history) {
         default:
@@ -530,7 +611,7 @@ static void option_menu_changed(
      * response to slider changes
      */
 
-    g_object_set_data(G_OBJECT(option_menu), "color_channel",
+    g_object_set_data(G_OBJECT(combo_box), "color_channel",
                       GINT_TO_POINTER(channel));
 
     for (i = 0; i < 3; i++) {
@@ -656,7 +737,7 @@ static void reset_button_clicked(
 )
 {
     CtkColorCorrection *ctk_color_correction;
-    GtkOptionMenu *option_menu;
+    GtkComboBox *combo_box;
     ctk_color_correction = CTK_COLOR_CORRECTION(user_data);
     /* Set default values */
     set_color_state(ctk_color_correction, CONTRAST, ALL_CHANNELS,
@@ -666,21 +747,23 @@ static void reset_button_clicked(
     set_color_state(ctk_color_correction, GAMMA, ALL_CHANNELS,
                     GAMMA_DEFAULT, TRUE);
 
+    ctk_color_correction->num_expected_updates++;
+    
     flush_attribute_channel_values(ctk_color_correction,
                                    ALL_VALUES, ALL_CHANNELS);
 
-    option_menu = GTK_OPTION_MENU(ctk_color_correction->option_menu);
+    combo_box = GTK_COMBO_BOX(ctk_color_correction->color_channel);
     
-    if (gtk_option_menu_get_history(option_menu) == 0) {
+    if (gtk_combo_box_get_active(combo_box) == 0) {
         /*
-         * gtk_option_menu_set_history will not emit the "changed" signal
-         * unless the new index differs from the old one; reasonable, but
-         * we need to cope with it here.
+         * We use the color_channel_changed function to reload color information
+         * from the server. If we are already on the correct channel, we cannot
+         * rely on the "changed" signal to be triggered so we will just call it
+         * directly here.
          */
-        gtk_option_menu_set_history(option_menu, 1);
-        gtk_option_menu_set_history(option_menu, 0);
+        color_channel_changed(combo_box, user_data);
     } else {
-        gtk_option_menu_set_history(option_menu, 0);
+        gtk_combo_box_set_active(combo_box, 0);
     }
 
     ctk_config_statusbar_message(ctk_color_correction->ctk_config,
@@ -717,11 +800,13 @@ static void adjustment_value_changed(
     attribute = GPOINTER_TO_INT(user_data);
 
     user_data = g_object_get_data
-        (G_OBJECT(ctk_color_correction->option_menu), "color_channel");
+        (G_OBJECT(ctk_color_correction->color_channel), "color_channel");
     channel = GPOINTER_TO_INT(user_data);
 
     value = gtk_adjustment_get_value(adjustment);
     
+    ctk_color_correction->num_expected_updates++;
+
     /* start timer for confirming changes */
     ctk_color_correction->confirm_countdown =
         DEFAULT_CONFIRM_COLORCORRECTION_TIMEOUT;
@@ -826,14 +911,16 @@ static void flush_attribute_channel_values(
     gint channel
 )
 {
-    NvCtrlAttributeHandle *handle = ctk_color_correction->handle;
-    
-    NvCtrlSetColorAttributes(handle,
+    CtrlTarget *ctrl_target = ctk_color_correction->ctrl_target;
+
+    NvCtrlSetColorAttributes(ctrl_target,
                              ctk_color_correction->cur_slider_val[CONTRAST],
                              ctk_color_correction->cur_slider_val[BRIGHTNESS],
                              ctk_color_correction->cur_slider_val[GAMMA],
                              attribute | channel);
-    
+
+    gtk_widget_hide(ctk_color_correction->warning_container);
+
     g_signal_emit(ctk_color_correction, signals[CHANGED], 0);
 }
 
@@ -843,9 +930,12 @@ static void apply_parsed_attribute_list(
     ParsedAttribute *p
 )
 {
+    CtrlTarget *ctrl_target = ctk_color_correction->ctrl_target;
     int target_type, target_id;
     unsigned int attr = 0;
-    
+
+    ctk_color_correction->num_expected_updates = 0;
+
     set_color_state(ctk_color_correction, CONTRAST, ALL_CHANNELS,
                     CONTRAST_DEFAULT, TRUE);
     set_color_state(ctk_color_correction, BRIGHTNESS, ALL_CHANNELS,
@@ -853,33 +943,48 @@ static void apply_parsed_attribute_list(
     set_color_state(ctk_color_correction, GAMMA, ALL_CHANNELS,
                     GAMMA_DEFAULT, TRUE);
 
-    target_type = NvCtrlGetTargetType(ctk_color_correction->handle);
-    target_id = NvCtrlGetTargetId(ctk_color_correction->handle);
+    target_type = NvCtrlGetTargetType(ctrl_target);
+    target_id = NvCtrlGetTargetId(ctrl_target);
 
     while (p) {
-        CtrlHandleTargetNode *node;
+        CtrlTargetNode *node;
+        const AttributeTableEntry *a = p->attr_entry;
 
         if (!p->next) goto next_attribute;
-        
-        if (!(p->flags & NV_PARSER_TYPE_COLOR_ATTRIBUTE)) goto next_attribute;
-        
+
+        if (a->type != CTRL_ATTRIBUTE_TYPE_COLOR) {
+            if (a->attr == NV_CTRL_COLOR_SPACE ||
+                a->attr == NV_CTRL_COLOR_RANGE) {
+                for (node = p->targets; node ; node = node->next) {
+                    int attr_target_type = NvCtrlGetTargetType(node->t);
+                    int attr_target_id = NvCtrlGetTargetId(node->t);
+
+                    if ((attr_target_type == target_type) &&
+                        (attr_target_id == target_id)) {
+                        ctk_color_correction->num_expected_updates++;
+                    }
+                }
+            }
+            goto next_attribute;
+        }
+
         /*
          * Apply the parsed attribute's settings only if the color 
          * correction's target matches one of the (parse attribute's)
          * specification targets.
          */
-        
-        for (node = p->targets; node && node->next; node = node->next) {
 
-            int attr_target_type = NvCtrlGetTargetType(node->t->h);
-            int attr_target_id = NvCtrlGetTargetId(node->t->h);
+        for (node = p->targets; node; node = node->next) {
+
+            int attr_target_type = NvCtrlGetTargetType(node->t);
+            int attr_target_id = NvCtrlGetTargetId(node->t);
 
             if ((attr_target_type != target_type) ||
                 (attr_target_id != target_id)) {
                 continue;
             }
-            
-            switch (p->attr & (ALL_VALUES | ALL_CHANNELS)) {
+
+            switch (a->attr & (ALL_VALUES | ALL_CHANNELS)) {
             case (CONTRAST_VALUE | RED_CHANNEL):
                 set_color_state(ctk_color_correction, CONTRAST,
                                 RED_CHANNEL, p->val.f, TRUE); break;
@@ -922,13 +1027,15 @@ static void apply_parsed_attribute_list(
             default:
                 continue;
             }
-        
-            attr |= (p->attr & (ALL_VALUES | ALL_CHANNELS));
+
+            ctk_color_correction->num_expected_updates++;
+
+            attr |= (a->attr & (ALL_VALUES | ALL_CHANNELS));
 
         }
-        
+
     next_attribute:
-        
+
         p = p->next;
     }
 
@@ -952,7 +1059,9 @@ static void apply_parsed_attribute_list(
             }
         }
 
-        NvCtrlSetColorAttributes(ctk_color_correction->handle,
+        ctk_color_correction->num_expected_updates++;
+
+        NvCtrlSetColorAttributes(ctrl_target,
                                  ctk_color_correction->cur_slider_val[CONTRAST],
                                  ctk_color_correction->cur_slider_val[BRIGHTNESS],
                                  ctk_color_correction->cur_slider_val[GAMMA],
@@ -990,8 +1099,8 @@ static gboolean do_confirm_countdown(gpointer data)
     CtkColorCorrection *ctk_color_correction = (CtkColorCorrection *)(data);
     unsigned int channels = 0;
     unsigned int attributes = 0;
-    GtkOptionMenu *option_menu =
-        GTK_OPTION_MENU(ctk_color_correction->option_menu);
+    GtkComboBox *combo_box =
+        GTK_COMBO_BOX(ctk_color_correction->color_channel);
 
     ctk_color_correction->confirm_countdown--;
     if (ctk_color_correction->confirm_countdown > 0) {
@@ -999,6 +1108,8 @@ static gboolean do_confirm_countdown(gpointer data)
         return True;
     }
 
+    ctk_color_correction->num_expected_updates++;
+
     /* Countdown timed out, reset color settings to previous state */
     for (attr = CONTRAST_INDEX; attr <= GAMMA_INDEX; attr++) {
         for (ch = RED; ch <= ALL_CHANNELS_INDEX; ch++) {
@@ -1019,7 +1130,7 @@ static gboolean do_confirm_countdown(gpointer data)
     }
     
     /* Refresh color correction page for current selected channel. */
-    option_menu_changed(option_menu, (gpointer)(ctk_color_correction));
+    color_channel_changed(combo_box, (gpointer)(ctk_color_correction));
     
     /* Reset confirm button text */
     gtk_label_set_text(GTK_LABEL(ctk_color_correction->confirm_label),
@@ -1053,7 +1164,7 @@ GtkTextBuffer *ctk_color_correction_create_help(GtkTextTagTable *table)
     ctk_color_correction_tab_help(b, &i, title, FALSE /* randr */);
     
     ctk_help_heading(b, &i, "Reset Hardware Defaults");
-    ctk_help_para(b, &i, __resest_button_help);
+    ctk_help_para(b, &i, "%s", __resest_button_help);
 
     ctk_help_finish(b);
 
@@ -1068,7 +1179,7 @@ void ctk_color_correction_tab_help(GtkTextBuffer *b, GtkTextIter *i,
     ctk_help_heading(b, i, "Color Correction");
 
     ctk_help_term(b, i, "Active Color Channel");
-    ctk_help_para(b, i, __active_color_help);
+    ctk_help_para(b, i, "%s", __active_color_help);
 
     ctk_help_term(b, i, "Brightness, Contrast and Gamma");
     ctk_help_para(b, i, "The Brightness, Contrast and Gamma sliders "
@@ -1087,7 +1198,7 @@ void ctk_color_correction_tab_help(GtkTextBuffer *b, GtkTextIter *i,
                   "value equally across all channels will make these games "
                   "appear brighter, making them more playable.");
 
-    ctk_help_para(b, i, __color_curve_help);
+    ctk_help_para(b, i, "%s", __color_curve_help);
 
     if (randr) {
         ctk_help_para(b, i, "The %s tab uses the RandR extension to "
@@ -1098,5 +1209,33 @@ void ctk_color_correction_tab_help(GtkTextBuffer *b, GtkTextIter *i,
     }
 
     ctk_help_term(b, i, "Confirm Current Changes");
-    ctk_help_para(b, i, __confirm_button_help);
+    ctk_help_para(b, i, "%s", __confirm_button_help);
 }
+
+
+static void callback_palette_update(GObject *object,
+                                    CtrlEvent *event,
+                                    gpointer user_data)
+{
+    gboolean reload_needed;
+    CtkColorCorrection *ctk_color_correction = (CtkColorCorrection *)user_data;
+    CtrlTarget *ctrl_target = ctk_color_correction->ctrl_target;
+
+    reload_needed = (ctk_color_correction->num_expected_updates <= 0);
+
+    if (ctk_color_correction->num_expected_updates > 0) {
+        ctk_color_correction->num_expected_updates--;
+    }
+
+    if (reload_needed) {
+        NvCtrlReloadColorRamp(ctrl_target);
+
+        ctk_curve_color_changed(ctk_color_correction->curve);
+        gtk_widget_set_sensitive(ctk_color_correction->reset_button, TRUE);
+
+        gtk_widget_show(ctk_color_correction->warning_container);
+    } else {
+        gtk_widget_hide(ctk_color_correction->warning_container);
+    }
+}
+
