@@ -112,6 +112,7 @@ static GridConfig gridConfigs[] = {
     {2, 2,  FALSE},
     {2, 3,  FALSE},
     {2, 4,  FALSE},
+    {4, 2,  FALSE},
     {3, 1,  FALSE},
     {3, 2,  FALSE},
     {1, 3,  FALSE},
@@ -119,6 +120,10 @@ static GridConfig gridConfigs[] = {
     {1, 2,  FALSE},
     {4, 1,  FALSE},
     {1, 4,  FALSE},
+    {4, 8,  FALSE},
+    {8, 4,  FALSE},
+    {2, 16, FALSE},
+    {16, 2, FALSE},
 };
 
 GType ctk_slimm_get_type()
@@ -499,7 +504,7 @@ static void slimm_checkbox_toggled(GtkWidget *widget, gpointer user_data)
         gtk_widget_set_sensitive(ctk_object->box_total_size, True);
         set_overlap_controls_status(ctk_object);
     } else {
-        if (GTK_WIDGET_SENSITIVE(ctk_object->mnu_display_refresh)) {
+        if (ctk_widget_get_sensitive(ctk_object->mnu_display_refresh)) {
             ctk_object->mnu_refresh_disabled = True;
             gtk_widget_set_sensitive(ctk_object->mnu_display_refresh, False);
         }
@@ -577,12 +582,12 @@ static void setup_display_refresh_dropdown(CtkSLIMM *ctk_object)
     /* Generate the refresh dropdown */
     menu = CTK_DROP_DOWN_MENU(ctk_object->mnu_display_refresh);
     
-    ctk_drop_down_menu_reset(menu);
-
     g_signal_handlers_block_by_func
         (G_OBJECT(ctk_object->mnu_display_refresh),
          G_CALLBACK(display_refresh_changed), (gpointer) ctk_object);
 
+    ctk_drop_down_menu_reset(menu);
+
     /* Generate the refresh rate dropdown from the modelines list */
     for (modeline = ctk_object->modelines; modeline; modeline = modeline->next) {
 
@@ -843,6 +848,7 @@ static Bool parse_slimm_layout(CtkSLIMM *ctk_slimm,
                                int *voverlap,
                                int *grid_config_id)
 {
+    CtrlTarget *ctrl_target = ctk_slimm->ctrl_target;
     ReturnStatus ret;
     char *metamode_str = NULL;
     char *str;
@@ -898,7 +904,7 @@ static Bool parse_slimm_layout(CtkSLIMM *ctk_slimm,
 
 
     /* Get the current metamode string */
-    ret = NvCtrlGetStringAttribute(ctk_slimm->handle,
+    ret = NvCtrlGetStringAttribute(ctrl_target,
                                    NV_CTRL_STRING_CURRENT_METAMODE,
                                    &metamode_str);
     if ((ret != NvCtrlSuccess) || !metamode_str) {
@@ -1108,8 +1114,8 @@ static Bool parse_slimm_layout(CtkSLIMM *ctk_slimm,
             *hoverlap += (*cur_modeline)->data.hdisplay;
         }
     }
-    
-    XFree(metamode_str);
+
+    free(metamode_str);
     return TRUE;
 
 
@@ -1117,16 +1123,14 @@ static Bool parse_slimm_layout(CtkSLIMM *ctk_slimm,
     *hoverlap = 0;
     *voverlap = 0;
     *grid_config_id = 0;
-    
+
     if (err_msg) {
         nv_warning_msg("Unable to determine current SLI Mosaic Mode "
                        "configuration (will fall back to default): %s\n",
                        err_msg);
     }
 
-    if (metamode_str) {
-        XFree(metamode_str);
-    }
+    free(metamode_str);
 
     return FALSE;
 }
@@ -1259,9 +1263,23 @@ static nvDisplayPtr intersect_modelines(nvLayoutPtr layout)
     return display;
 }
 
+#define STEREO_IS_3D_VISION(stereo) \
+    (((stereo) == NV_CTRL_STEREO_3D_VISION) || \
+     ((stereo) == NV_CTRL_STEREO_3D_VISION_PRO))
 
-GtkWidget* ctk_slimm_new(NvCtrlAttributeHandle *handle,
-                          CtkEvent *ctk_event, CtkConfig *ctk_config)
+static int get_display_stereo_mode(nvDisplayPtr display)
+{
+    if ((display->screen == NULL) ||
+        (display->screen->stereo_supported == FALSE)) {
+        return NV_CTRL_STEREO_OFF;
+    } else {
+        return display->screen->stereo;
+    }
+}
+
+GtkWidget* ctk_slimm_new(CtrlTarget *ctrl_target,
+                         CtkEvent *ctk_event,
+                         CtkConfig *ctk_config)
 {
     GObject *object;
     CtkSLIMM *ctk_slimm;
@@ -1279,6 +1297,7 @@ GtkWidget* ctk_slimm_new(NvCtrlAttributeHandle *handle,
     CtkDropDownMenu *menu;
 
     gchar *err_str = NULL;
+    gchar *str;
     gchar *tmp;
     gchar *sli_mode = NULL;
     ReturnStatus ret;
@@ -1294,20 +1313,17 @@ GtkWidget* ctk_slimm_new(NvCtrlAttributeHandle *handle,
     int count;
 
     Bool trust_slimm_available = FALSE;
-    int vcs_target_count;
 
     int hoverlap = 0;
     int voverlap = 0;
     int grid_config_id = 0;
 
     /* now, create the object */
-    
+
     object = g_object_new(CTK_TYPE_SLIMM, NULL);
     ctk_slimm = CTK_SLIMM(object);
 
-    /* cache the attribute handle */
-
-    ctk_slimm->handle = handle;
+    ctk_slimm->ctrl_target = ctrl_target;
     ctk_slimm->ctk_config = ctk_config;
     ctk_object = ctk_slimm;
 
@@ -1316,41 +1332,40 @@ GtkWidget* ctk_slimm_new(NvCtrlAttributeHandle *handle,
      * This is used to not trust old X drivers which always reported
      * it available (on NV50+).
      */
-    ret = NvCtrlGetAttribute(handle,
+    ret = NvCtrlGetAttribute(ctrl_target,
                              NV_CTRL_ATTR_NV_MAJOR_VERSION, &major);
-    ret1 = NvCtrlGetAttribute(handle,
+    ret1 = NvCtrlGetAttribute(ctrl_target,
                               NV_CTRL_ATTR_NV_MINOR_VERSION, &minor);
 
     if ((ret == NvCtrlSuccess) && (ret1 == NvCtrlSuccess) &&
         ((major > 1) || ((major == 1) && (minor > 23)))) {
         trust_slimm_available = TRUE;
     }
-    
-    vcs_target_count = ctk_config->pCtrlHandles->targets[VCS_TARGET].n;
 
     /* return on old X drivers if target is other than VCS. */
-    if (!vcs_target_count && !trust_slimm_available) {
+    if (!ctk_config->pCtrlSystem->targets[VCS_TARGET] &&
+        !trust_slimm_available) {
       return NULL;
     }
 
     /* Check if this screen supports SLI Mosaic Mode */
-    ret = NvCtrlGetAttribute(ctk_object->handle,
+    ret = NvCtrlGetAttribute(ctrl_target,
                              NV_CTRL_SLI_MOSAIC_MODE_AVAILABLE, &val);
-    if ((ret == NvCtrlSuccess) && 
+    if ((ret == NvCtrlSuccess) &&
         (val == NV_CTRL_SLI_MOSAIC_MODE_AVAILABLE_FALSE)) {
         /* Mosaic not supported */
         return NULL;
     }
 
     /* Query the maximum screen sizes */
-    ret = NvCtrlGetAttribute(ctk_object->handle,
+    ret = NvCtrlGetAttribute(ctrl_target,
                              NV_CTRL_MAX_SCREEN_WIDTH,
                              &ctk_slimm->max_screen_width);
     if (ret != NvCtrlSuccess) {
         return NULL;
     }
 
-    ret = NvCtrlGetAttribute(ctk_object->handle,
+    ret = NvCtrlGetAttribute(ctrl_target,
                              NV_CTRL_MAX_SCREEN_HEIGHT,
                              &ctk_slimm->max_screen_height);
     if (ret != NvCtrlSuccess) {
@@ -1363,7 +1378,7 @@ GtkWidget* ctk_slimm_new(NvCtrlAttributeHandle *handle,
      */
 
     /* Load the layout structure from the X server */
-    layout = layout_load_from_server(handle, &err_str);
+    layout = layout_load_from_server(ctrl_target, &err_str);
 
     if (!err_str && layout) {
         nvGpuPtr gpu;
@@ -1415,41 +1430,66 @@ GtkWidget* ctk_slimm_new(NvCtrlAttributeHandle *handle,
 
     /* If we failed to load, tell the user why */
     if (err_str || !layout) {
-        gchar *str;
-
-        if (!err_str) {
-            str = g_strdup("Unable to load SLI Mosaic Mode Settings page.");
-        } else {
-            str = g_strdup_printf("Unable to load SLI Mosaic Mode Settings "
-                                  "page:\n\n%s", err_str);
-            g_free(err_str);
-        }
-
-        label = gtk_label_new(str);
-        g_free(str);
-        gtk_label_set_selectable(GTK_LABEL(label), TRUE);
-        gtk_container_add(GTK_CONTAINER(object), label);
-
-        /* Show the GUI */
-        gtk_widget_show_all(GTK_WIDGET(ctk_object));
-
-        return GTK_WIDGET(ctk_object);
+        goto slimm_fail;
     }
 
     display = intersect_modelines(layout);
 
     if (display == NULL) {
-        gchar *str = g_strdup("Unable to find active display with "
-                              "intersected modelines.");
-        label = gtk_label_new(str);
-        g_free(str);
-        gtk_label_set_selectable(GTK_LABEL(label), TRUE);
-        gtk_container_add(GTK_CONTAINER(object), label);
+        err_str = g_strdup("Unable to find active display with "
+                           "intersected modelines.");
+        goto slimm_fail;
+    } else if ((display->modelines == NULL) &&
+               (display->cur_mode->modeline == NULL)) {
+        /* The modepool for the active display did not have any modes in
+         * its modepool matching any of the modes on the modepool of any
+         * other display in the layout, causing intersect_modelines to
+         * remove every mode from the list of available modes for SLI mosaic
+         * mode.
+         *
+         * This can happen if one display had its modepool trimmed and modified
+         * to support 3D vision, while other displays (either on X screens
+         * without stereo currently enabled, or on screenless GPUs) did not.
+         * Find if that is the case, and display an informative message if so.
+         */
+        nvGpuPtr gpu;
+        nvDisplayPtr d;
+        int stereo = get_display_stereo_mode(display);
 
-        /* Show the GUI */
-        gtk_widget_show_all(GTK_WIDGET(ctk_object));
+        for (gpu = layout->gpus; gpu; gpu = gpu->next_in_layout) {
+            for (d = gpu->displays; d; d = d->next_on_gpu) {
+                int other_stereo;
 
-        return GTK_WIDGET(ctk_object);
+                if (display == d) {
+                    continue;
+                }
+
+                other_stereo = get_display_stereo_mode(d);
+
+                if ((STEREO_IS_3D_VISION(stereo) &&
+                    !STEREO_IS_3D_VISION(other_stereo)) ||
+                    (!STEREO_IS_3D_VISION(stereo) &&
+                     STEREO_IS_3D_VISION(other_stereo))) {
+
+                    err_str = g_strdup("Unable to find common modelines between\n"
+                                       "all connected displays due to 3D vision\n"
+                                       "being enabled on some displays and not\n"
+                                       "others. Please make sure that 3D vision\n"
+                                       "is enabled on all connected displays\n"
+                                       "before enabling SLI mosaic mode.");
+
+                    goto slimm_fail;
+                }
+            }
+        }
+
+        /* The intersected modepool was empty, but not because of a mismatch
+         * in 3D Vision settings.
+         */
+        err_str = g_strdup("Unable to find find common modelines between "
+                           "all connected displays.");
+
+        goto slimm_fail;
     }
 
 
@@ -1457,11 +1497,8 @@ GtkWidget* ctk_slimm_new(NvCtrlAttributeHandle *handle,
     ctk_object->modelines = display->modelines;
     if (display->cur_mode->modeline) {
         ctk_object->cur_modeline = display->cur_mode->modeline; 
-    } else if (ctk_object->modelines) {
-        ctk_object->cur_modeline = ctk_object->modelines;
     } else {
-        /* This is an error. */
-        return NULL;
+        ctk_object->cur_modeline = ctk_object->modelines;
     }
     ctk_object->num_modelines = display->num_modelines;
 
@@ -1509,7 +1546,7 @@ GtkWidget* ctk_slimm_new(NvCtrlAttributeHandle *handle,
 
     /* Option menu for Display Grid Configuration */
     menu = (CtkDropDownMenu *)
-        ctk_drop_down_menu_new(CTK_DROP_DOWN_MENU_FLAG_COMBO);
+        ctk_drop_down_menu_new(CTK_DROP_DOWN_MENU_FLAG_READONLY);
     ctk_slimm->mnu_display_config = GTK_WIDGET(menu);
 
     grid_menu_selected_id = 0;
@@ -1570,7 +1607,7 @@ GtkWidget* ctk_slimm_new(NvCtrlAttributeHandle *handle,
     /* Option menu for resolutions */
     hbox = gtk_hbox_new(FALSE, 0);
     ctk_slimm->mnu_display_resolution =
-        ctk_drop_down_menu_new(CTK_DROP_DOWN_MENU_FLAG_COMBO);
+        ctk_drop_down_menu_new(CTK_DROP_DOWN_MENU_FLAG_READONLY);
 
     /* Create a drop down menu */
     setup_display_resolution_dropdown(ctk_object);
@@ -1588,7 +1625,7 @@ GtkWidget* ctk_slimm_new(NvCtrlAttributeHandle *handle,
     /* Option menu for refresh rates */
     hbox = gtk_hbox_new(FALSE, 0);
     ctk_slimm->mnu_display_refresh =
-        ctk_drop_down_menu_new(CTK_DROP_DOWN_MENU_FLAG_COMBO);
+        ctk_drop_down_menu_new(CTK_DROP_DOWN_MENU_FLAG_READONLY);
     setup_display_refresh_dropdown(ctk_object);
     g_signal_connect(G_OBJECT(ctk_object->mnu_display_refresh), "changed",
                      G_CALLBACK(display_refresh_changed),
@@ -1718,7 +1755,7 @@ GtkWidget* ctk_slimm_new(NvCtrlAttributeHandle *handle,
                      GTK_EXPAND | GTK_FILL, 0, 0);
 
     /* If current SLI Mode != Mosaic, disable UI elements initially */
-    ret = NvCtrlGetStringAttribute(ctk_slimm->handle,
+    ret = NvCtrlGetStringAttribute(ctrl_target,
                                    NV_CTRL_STRING_SLI_MODE,
                                    &sli_mode);
 
@@ -1730,13 +1767,35 @@ GtkWidget* ctk_slimm_new(NvCtrlAttributeHandle *handle,
         slimm_checkbox_toggled(ctk_slimm->cbtn_slimm_enable, (gpointer) ctk_slimm);
     }
 
-    if (sli_mode) {
-        XFree(sli_mode);
-    }
+    free(sli_mode);
 
     gtk_widget_show_all(GTK_WIDGET(object));    
 
     return GTK_WIDGET(object);
+
+slimm_fail:
+
+    if (layout) {
+        layout_free(layout);
+    }
+
+    if (!err_str) {
+        str = g_strdup("Unable to load SLI Mosaic Mode Settings page.");
+    } else {
+        str = g_strdup_printf("Unable to load SLI Mosaic Mode Settings "
+                              "page:\n\n%s", err_str);
+        g_free(err_str);
+    }
+
+    label = gtk_label_new(str);
+    g_free(str);
+    gtk_label_set_selectable(GTK_LABEL(label), TRUE);
+    gtk_container_add(GTK_CONTAINER(object), label);
+
+    /* Show the GUI */
+    gtk_widget_show_all(GTK_WIDGET(ctk_object));
+
+    return GTK_WIDGET(ctk_object);
 }
 
 
@@ -1792,6 +1851,10 @@ GtkTextBuffer *ctk_slimm_create_help(GtkTextTagTable *table,
     ctk_help_para(b, &i, "This is the total size of the X screen formed using all "
                   "displays in SLI Mosaic Mode.");
 
+    ctk_help_heading(b, &i, "Maximum Size");
+    ctk_help_para(b, &i, "This is the maximum allowable size of the X screen "
+                  "formed using all displays in SLI Mosaic Mode.");
+
     ctk_help_heading(b, &i, "Save to X Configuration File");
     ctk_help_para(b, &i, "Clicking this button saves the selected SLI Mosaic Mode "
                   "settings into the X Configuration File.");
