@@ -514,14 +514,14 @@ Bool XNVCTRLSetTargetStringAttribute (
     int target_id,
     unsigned int display_mask,
     unsigned int attribute,
-    char *ptr
+    const char *ptr
 ){
     XExtDisplayInfo *info = find_display (dpy);
     xnvCtrlSetStringAttributeReq *req;
     xnvCtrlSetStringAttributeReply rep;
     int size;
     Bool success;
-    
+
     if(!XextHasExtension(info))
         return False;
 
@@ -540,7 +540,7 @@ Bool XNVCTRLSetTargetStringAttribute (
     req->length += ((size + 3) & ~3) >> 2;
     req->num_bytes = size;
     Data(dpy, ptr, size);
-    
+
     if (!_XReply (dpy, (xReply *) &rep, 0, False)) {
         UnlockDisplay (dpy);
         SyncHandle ();
@@ -548,7 +548,7 @@ Bool XNVCTRLSetTargetStringAttribute (
     }
     UnlockDisplay (dpy);
     SyncHandle ();
-    
+
     success = rep.flags;
     return success;
 }
@@ -558,7 +558,7 @@ Bool XNVCTRLSetStringAttribute (
     int screen,
     unsigned int display_mask,
     unsigned int attribute,
-    char *ptr
+    const char *ptr
 ){
     return XNVCTRLSetTargetStringAttribute(dpy, NV_CTRL_TARGET_TYPE_X_SCREEN,
                                            screen, display_mask,
@@ -1059,7 +1059,7 @@ Bool XNVCTRLStringOperation (
     int target_id,
     unsigned int display_mask,
     unsigned int attribute,
-    char *pIn,
+    const char *pIn,
     char **ppOut
 ) {
     XExtDisplayInfo *info = find_display(dpy);
@@ -1070,24 +1070,24 @@ Bool XNVCTRLStringOperation (
 
     if (!XextHasExtension(info))
         return False;
-    
+
     if (!ppOut)
         return False;
 
     *ppOut = NULL;
-    
+
     XNVCTRLCheckExtension(dpy, info, False);
     XNVCTRLCheckTargetData(dpy, info, &target_type, &target_id);
-    
+
     if (pIn) {
         inSize = strlen(pIn) + 1;
     } else {
         inSize = 0;
     }
-    
+
     LockDisplay(dpy);
     GetReq(nvCtrlStringOperation, req);
-    
+
     req->reqType = info->codes->major_opcode;
     req->nvReqType = X_nvCtrlStringOperation;
     req->target_type = target_type;
@@ -1097,38 +1097,81 @@ Bool XNVCTRLStringOperation (
 
     req->length += ((inSize + 3) & ~3) >> 2;
     req->num_bytes = inSize;
-    
+
     if (pIn) {
         Data(dpy, pIn, inSize);
     }
-    
+
     if (!_XReply (dpy, (xReply *) &rep, 0, False)) {
         UnlockDisplay(dpy);
         SyncHandle();
         return False;
     }
-    
+
     length = rep.length;
     outSize = rep.num_bytes;
     slop = outSize & 3;
 
     if (outSize) *ppOut = (char *) Xmalloc(outSize);
-    
+
     if (!*ppOut) {
         _XEatData(dpy, length);
     } else {
         _XRead(dpy, (char *) *ppOut, outSize);
         if (slop) _XEatData(dpy, 4-slop);
     }
-    
+
     ret = rep.ret;
-    
+
     UnlockDisplay(dpy);
     SyncHandle();
-    
+
     return ret;
 }
 
+Bool XNVCTRLBindWarpPixmapName (
+    Display *dpy,
+    int screen,
+    Pixmap pix,
+    const char *name,
+    unsigned int dataType,
+    unsigned int vertexCount
+) {
+    XExtDisplayInfo *info = find_display(dpy);
+    xnvCtrlBindWarpPixmapNameReq *req;
+    unsigned int nameLen;
+
+    if (!XextHasExtension(info))
+        return False;
+
+    if (!name)
+        return False;
+
+    nameLen = strlen(name) + 1;
+
+    XNVCTRLCheckExtension(dpy, info, False);
+
+    LockDisplay(dpy);
+    GetReq(nvCtrlBindWarpPixmapName, req);
+
+    req->reqType = info->codes->major_opcode;
+    req->nvReqType = X_nvCtrlBindWarpPixmapName;
+
+    req->length += ((nameLen + 3) & ~3) >> 2;
+    req->num_bytes = nameLen;
+
+    req->screen = screen;
+    req->pixmap = pix;
+    req->dataType = dataType;
+    req->vertexCount = vertexCount;
+
+    Data(dpy, name, nameLen);
+
+    UnlockDisplay(dpy);
+    SyncHandle();
+
+    return True;
+}
 
 static Bool wire_to_event (Display *dpy, XEvent *host, xEvent *wire)
 {
